/**
 * Copyright (c) 2020, Oracle and/or its affiliates.
 * Licensed under the Universal Permissive License v 1.0 as shown at http://oss.oracle.com/licenses/upl.
 */
define(['ojs/ojcore',
    'knockout',
    'jquery',
    'viewModels/common_vm_def',
    'models/administration',
    'viewModels/confirmation-dialog',
    'models/platform',
    'ojs/ojinputtext',
    'ojs/ojlabel',
    'ojs/ojdialog',
    'ojs/ojswitch',
    'ojs/ojvalidation-base',
    'ojs/ojdatetimepicker',
    'ojs/ojtimezonedata'],
    function (oj, ko, $, CommonVMDefinitons, adminSVC, confirmation, Platform) {
        function updateWindowViewModel() {
            var self = this;
            this.updateWindow = ko.observable();
            this.updateTime= ko.observable();

            this.open = function () {
                new adminSVC().getUpdateTimeInfo().then(function (data) {
                    self.updateTime(data[0]);
                    self.updateWindow(getTimeWindow(data[0], data[1]).toString());
                    $(self.dialogSelector).ojDialog('open');
                }).catch(function (why) {
                    oj.Logger.error('Failed to retrieve update time window');
                    confirmation.showDialog(
                        {
                            'notifyOnly': true,
                            'title': oj.Translations.getTranslatedString('Error'),
                            'message': oj.Translations.getTranslatedString('Failed to retrieve autonomous daily update time information: ' + why)
                        });
                }
                );
            };

            function getTimeWindow(startTime, endTime) {
                startTime = Number(startTime);
                endTime = Number(endTime);
                if (endTime > startTime)
                    return endTime - startTime;
                
                return (endTime + 24) - startTime;
            };

            function getEndTime(startTime, updateWindow) {
                var endTime = Number(startTime) + Number(updateWindow);
                if (endTime > 23)
                    endTime -= 24;
                return endTime;
            };

            this.handleCancelClick = function () {
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleCloseDrawerClick = function () {
                $(self.dialogSelector).ojDialog("close");
            };

            this.handleUpdateClick = function (data, event) {
                confirmation.showDialog({
                    'title': oj.Translations.getTranslatedString('Autonomous Linux daily update time window'),
                    'message': oj.Translations.getTranslatedString('Are you sure you want to update the time window ?'),
                    'onOK': function () {
                        new adminSVC().setUpdateTimeInfo(self.updateTime(), getEndTime(self.updateTime(), self.updateWindow())).then(function (payload) {
                            oj.Logger.info('Time window was updated successfully');
                            confirmation.showDialog({
                                'notifyOnly': true,
                                'title': oj.Translations.getTranslatedString('Successful Modification'),
                                'message': oj.Translations.getTranslatedString('Time window was updated successfully')
                            });

                        }).catch(function (why) {
                            oj.Logger.error('Updating Autonomous time window failed: ' + why);
                            confirmation.showDialog({
                                'notifyOnly': true,
                                'title': oj.Translations.getTranslatedString('Autonomous time window modification failure'),
                                'message': why
                            });
                        }
                        );
                        $(self.dialogSelector).ojDialog("close");
                    }
                });
            };
        }

        $.extend(updateWindowViewModel.prototype, CommonVMDefinitons.prototype.definitions);

        updateWindowViewModel.prototype.dialogSelector = '#updateWindowDialog';
        updateWindowViewModel.prototype.dialogTitle=oj.Translations.getTranslatedString('Update Autonomous Linux daily update time window');
        updateWindowViewModel.prototype.updateTimeLabel = oj.Translations.getTranslatedString('Start Time');
        updateWindowViewModel.prototype.updateTimeHelp = oj.Translations.getTranslatedString('Specify the start time for daily auto update window');
        updateWindowViewModel.prototype.updateTimeSource = 'https://community.oracle.com/docs/DOC-1034101';

        updateWindowViewModel.prototype.updateWindowLabel = oj.Translations.getTranslatedString('Update Window Duration');
        updateWindowViewModel.prototype.updateWindowHelp = oj.Translations.getTranslatedString('Length in hours of the update window');
        updateWindowViewModel.prototype.updateWindowSource = 'https://community.oracle.com/docs/DOC-1034101';

        updateWindowViewModel.prototype.updateWindowUpdateLabel = oj.Translations.getTranslatedString('Update');
        updateWindowViewModel.prototype.ocidValidationHintMessage = oj.Translations.getTranslatedString('OCID format should be similar to ocid1.onstopic.oc.....');

        return new updateWindowViewModel();
    }
);
